# -*- coding: utf-8 -*-
"""The nsinterface class.

$Id: nsinterface.py 50795 2008-02-21 10:58:27Z cray $
"""
__author__  = "Dima Khozin & Andrey Orlov"
__license__ = "GPL"
__version__ = "$Revision: 50795 $"
__date__ = "$Date: 2008-02-21 13:58:27 +0300 (Чтв, 21 Фев 2008) $"
__credits__ = """Andrey Orlov, for idea and common control"""

from zope.interface import implements
from zope.traversing.interfaces import ITraversable
from zope.component.interface import nameToInterface
from zope.traversing.namespace import SimpleHandler
from zope.security.proxy import removeSecurityProxy
from zope.traversing.interfaces import IPhysicallyLocatable
from zope.location import location

class Nsinterface(SimpleHandler):
    """ ++interface++"""

    implements(ITraversable)

    def traverse(self,name,ignored) :
        """алгоритм приведения контекстного объекта к нужному интерфейсу"""
        res = nameToInterface(self.context,name)(self.context)
        res = removeSecurityProxy(res)
        if True : #not IPhysicallyLocatable(res, False):
            res = location.LocationProxy(
                res, self.context, '++nsinterface++'+name)
            return res
        return res
        



### -*- coding: utf-8 -*- #############################################
#######################################################################
"""Adaptive URL view components

$Id: adaptiveurl.py 50795 2008-02-21 10:58:27Z cray $
"""
__author__  = "Andrey Orlov, 2008"
__license__ = "GPL"
__version__ = "$Revision: 50795 $"

from zope.interface import Interface
from zope.interface import implements,implementedBy
from zope.component import getMultiAdapter
from zope.publisher.browser import BrowserView
from interfaces import IAdaptiveURL, IAdaptiveURLRoot
from zope.traversing.browser.absoluteurl import _safe, _insufficientContext
from zope.proxy import sameProxiedObjects
import urllib

class DoAdapt(Exception) :
    """ Adaptation need """

def adaptiveURL(ob, request):
    return getMultiAdapter((ob, request), name="adaptive_url")()

class SiteAdaptiveURL(BrowserView):
    implements(IAdaptiveURL)

    def __unicode__(self):
        return urllib.unquote(self.__str__()).decode('utf-8')

    def __str__(self):
        context = self.context
        request = self.request

        if sameProxiedObjects(context, request.getVirtualHostRoot()):
            return request.getApplicationURL()
        elif request.getVirtualHostRoot() :
            raise DoAdapt

        url = request.getApplicationURL()
 
        try :
            url =+  self._getContextName(context) 
        except TypeError :
            pass
 
        return url                        


    def __call__(self) :
        return self.__str__()

    def _getContextName(self, context):
        name = getattr(context, '__name__', None)
        if name is None:
            raise TypeError(_insufficientContext)
        return '/' + urllib.quote(name.encode('utf-8'), _safe)        

class AdaptiveURL(SiteAdaptiveURL):

    def __str__(self):
        context = self.context
        request = self.request

        # The application URL contains all the namespaces that are at the
        # beginning of the URL, such as skins, virtual host specifications and
        # so on.
        if (context is None
            or sameProxiedObjects(context, request.getVirtualHostRoot())):
            return request.getApplicationURL()
        
        try :            
            urlroot=IAdaptiveURLRoot(context)
        except TypeError :
            url = self._up(context,request)
        else :
            try :
                url = self._up(context,request)                
            except DoAdapt :
                return urlroot.adaptive_url
            
        url += self._getContextName(context)
        return url

                                
    def _up(self, context, request) :
        container = getattr(context, '__parent__', None)
        if container is None:
            raise TypeError(_insufficientContext)

        return str(getMultiAdapter((container, request),name='adaptive_url'))



### -*- coding: utf-8 -*- #############################################
#######################################################################
"""Interfaces for the Zope 3 based AdaptiveURL  package

$Id: interfaces.py 50795 2008-02-21 10:58:27Z cray $
"""
__author__  = ""
__license__ = "GPL"
__version__ = "$Revision: 50795 $"
 
from zope.interface import Interface

from zope.schema import URI
                
class IAdaptiveURL(Interface):

    def __unicode__():
        """Returns the URL as a unicode string."""

    def __str__():
        """Returns an ASCII string with all unicode characters url quoted."""

    def __repr__():
        """Get a string representation """

    def __call__():
        """Returns an ASCII string with all unicode characters url quoted."""

class IAdaptiveURLRoot(Interface) :
    
    adaptive_url = URI( title = u"Site Root URL" )
        


### -*- coding: utf-8 -*- #############################################
#######################################################################
"""NULL ISublocations adapter may be use to deny event distribution
into object subtree 

$Id: nullsublocations.py 50795 2008-02-21 10:58:27Z cray $
"""
__author__  = "Andrey Orlov, 2008"
__license__ = "GPL"
__version__ = "$Revision: 50795 $"

from zope.interface import Interface, implements
from zope.location.interfaces import ISublocations

class NullSublocations(object):
    """Get the null sublocations for a object """
    implements(ISublocations)

    def __init__(self, context):
        self.context = context

    def sublocations(self):
        if False :
            yield None
